import logging
from typing import Any, Collection

from fi_instrumentation.instrumentation.config import FITracer, TraceConfig
from opentelemetry import trace as trace_api
from opentelemetry.instrumentation.instrumentor import (  # type: ignore[attr-defined]
    BaseInstrumentor,
)
from traceai_anthropic._wrappers import (
    _AsyncCompletionsWrapper,
    _AsyncMessagesWrapper,
    _CompletionsWrapper,
    _MessagesCountTokensWrapper,
    _MessagesWrapper,
)
from traceai_anthropic.version import __version__
from wrapt import wrap_function_wrapper

logger = logging.getLogger(__name__)

_instruments = ("anthropic >= 0.30.0",)


class AnthropicInstrumentor(BaseInstrumentor):  # type: ignore[misc]
    """An instrumentor for the Anthropic framework."""

    __slots__ = (
        "_original_completions_create",
        "_original_async_completions_create",
        "_original_messages_create",
        "_original_async_messages_create",
        "_instruments",
        "_tracer",
    )

    def instrumentation_dependencies(self) -> Collection[str]:
        return _instruments

    def _instrument(self, **kwargs: Any) -> None:
        from anthropic.resources.completions import AsyncCompletions, Completions
        from anthropic.resources.messages import AsyncMessages, Messages

        if not (tracer_provider := kwargs.get("tracer_provider")):
            tracer_provider = trace_api.get_tracer_provider()
        if not (config := kwargs.get("config")):
            config = TraceConfig()
        else:
            assert isinstance(config, TraceConfig)

        self._tracer = FITracer(
            trace_api.get_tracer(__name__, __version__, tracer_provider),
            config=config,
        )

        self._original_completions_create = Completions.create
        wrap_function_wrapper(
            module="anthropic.resources.completions",
            name="Completions.create",
            wrapper=_CompletionsWrapper(tracer=self._tracer),
        )

        self._original_async_completions_create = AsyncCompletions.create
        wrap_function_wrapper(
            module="anthropic.resources.completions",
            name="AsyncCompletions.create",
            wrapper=_AsyncCompletionsWrapper(tracer=self._tracer),
        )

        self._original_messages_create = Messages.create
        wrap_function_wrapper(
            module="anthropic.resources.messages",
            name="Messages.create",
            wrapper=_MessagesWrapper(tracer=self._tracer),
        )

        self._original_async_messages_create = AsyncMessages.create
        wrap_function_wrapper(
            module="anthropic.resources.messages",
            name="AsyncMessages.create",
            wrapper=_AsyncMessagesWrapper(tracer=self._tracer),
        )

        wrap_function_wrapper(
            module="anthropic.resources.messages",
            name="Messages.count_tokens",
            wrapper=_MessagesCountTokensWrapper(tracer=self._tracer),
        )

    def _uninstrument(self, **kwargs: Any) -> None:
        from anthropic.resources.completions import AsyncCompletions, Completions
        from anthropic.resources.messages import AsyncMessages, Messages

        if self._original_completions_create is not None:
            Completions.create = self._original_completions_create  # type: ignore[method-assign]
        if self._original_async_completions_create is not None:
            AsyncCompletions.create = self._original_async_completions_create  # type: ignore[method-assign]

        if self._original_messages_create is not None:
            Messages.create = self._original_messages_create  # type: ignore[method-assign]
        if self._original_async_messages_create is not None:
            AsyncMessages.create = self._original_async_messages_create  # type: ignore[method-assign]



from collections import defaultdict
from copy import deepcopy
from typing import (
    TYPE_CHECKING,
    Any,
    AsyncIterator,
    Callable,
    DefaultDict,
    Dict,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Tuple,
    Union,
)

from fi_instrumentation import safe_json_dumps
from fi_instrumentation.fi_types import (
    FiMimeTypeValues,
    MessageAttributes,
    SpanAttributes,
    ToolCallAttributes,
)
from opentelemetry import trace as trace_api
from opentelemetry.util.types import AttributeValue
from traceai_anthropic._utils import (
    _as_output_attributes,
    _finish_tracing,
    _ValueAndType,
)
from traceai_anthropic._with_span import _WithSpan
from wrapt import ObjectProxy

if TYPE_CHECKING:
    from anthropic import Stream
    from anthropic.types import Completion, RawMessageStreamEvent
    from anthropic.types.raw_content_block_delta_event import RawContentBlockDeltaEvent
    from anthropic.types.text_block import TextBlock
    from anthropic.types.tool_use_block import ToolUseBlock


class _Stream(ObjectProxy):  # type: ignore
    __slots__ = (
        "_response_accumulator",
        "_with_span",
        "_is_finished",
    )

    def __init__(
        self,
        stream: "Stream[Completion]",
        with_span: _WithSpan,
    ) -> None:
        super().__init__(stream)
        self._response_accumulator = _ResponseAccumulator()
        self._with_span = with_span

    def __iter__(self) -> Iterator["Completion"]:
        try:
            for item in self.__wrapped__:
                self._response_accumulator.process_chunk(item)
                yield item
        except Exception as exception:
            status = trace_api.Status(
                status_code=trace_api.StatusCode.ERROR,
                description=f"{type(exception).__name__}: {exception}",
            )
            self._with_span.record_exception(exception)
            self._finish_tracing(status=status)
            raise
        status = trace_api.Status(
            status_code=trace_api.StatusCode.OK,
        )
        self._finish_tracing(status=status)

    async def __aiter__(self) -> AsyncIterator["Completion"]:
        try:
            async for item in self.__wrapped__:
                self._response_accumulator.process_chunk(item)
                yield item
        except Exception as exception:
            status = trace_api.Status(
                status_code=trace_api.StatusCode.ERROR,
                description=f"{type(exception).__name__}: {exception}",
            )
            self._with_span.record_exception(exception)
            self._finish_tracing(status=status)
            raise
        # completed without exception
        status = trace_api.Status(
            status_code=trace_api.StatusCode.OK,
        )
        self._finish_tracing(status=status)

    def _finish_tracing(
        self,
        status: Optional[trace_api.Status] = None,
    ) -> None:
        _finish_tracing(
            with_span=self._with_span,
            has_attributes=_ResponseExtractor(
                response_accumulator=self._response_accumulator
            ),
            status=status,
        )


class _ResponseAccumulator:
    __slots__ = (
        "_is_null",
        "_values",
    )

    def __init__(self) -> None:
        self._is_null = True
        self._values = _ValuesAccumulator(
            completion=_StringAccumulator(),
            stop=_SimpleStringReplace(),
            stop_reason=_SimpleStringReplace(),
        )

    def process_chunk(self, chunk: "Completion") -> None:
        self._is_null = False
        values = chunk.model_dump(exclude_unset=True, warnings=False)
        self._values += values

    def _result(self) -> Optional[Dict[str, Any]]:
        if self._is_null:
            return None
        return dict(self._values)


class _ResponseExtractor:
    __slots__ = ("_response_accumulator",)

    def __init__(
        self,
        response_accumulator: _ResponseAccumulator,
    ) -> None:
        self._response_accumulator = response_accumulator

    def get_attributes(self) -> Iterator[Tuple[str, AttributeValue]]:
        if not (result := self._response_accumulator._result()):
            return

        json_string = safe_json_dumps(result)
        yield SpanAttributes.RAW_OUTPUT, json_string

        yield from _as_output_attributes(
            _ValueAndType(json_string, FiMimeTypeValues.JSON)
        )

    def get_extra_attributes(self) -> Iterator[Tuple[str, AttributeValue]]:
        if not (result := self._response_accumulator._result()):
            return
        if completion := result.get("completion", ""):
            yield SpanAttributes.LLM_OUTPUT_MESSAGES, completion


class _MessagesStream(ObjectProxy):  # type: ignore
    __slots__ = (
        "_response_accumulator",
        "_with_span",
        "_is_finished",
    )

    def __init__(
        self,
        stream: "Stream[RawMessageStreamEvent]",
        with_span: _WithSpan,
    ) -> None:
        super().__init__(stream)
        self._response_accumulator = _MessageResponseAccumulator()
        self._with_span = with_span

    def __iter__(self) -> Iterator["RawMessageStreamEvent"]:
        try:
            for item in self.__wrapped__:
                self._response_accumulator.process_chunk(item)
                yield item
        except Exception as exception:
            status = trace_api.Status(
                status_code=trace_api.StatusCode.ERROR,
                description=f"{type(exception).__name__}: {exception}",
            )
            self._with_span.record_exception(exception)
            self._finish_tracing(status=status)
            raise
        status = trace_api.Status(
            status_code=trace_api.StatusCode.OK,
        )
        self._finish_tracing(status=status)

    async def __aiter__(self) -> AsyncIterator["RawMessageStreamEvent"]:
        try:
            async for item in self.__wrapped__:
                self._response_accumulator.process_chunk(item)
                yield item
        except Exception as exception:
            status = trace_api.Status(
                status_code=trace_api.StatusCode.ERROR,
                description=f"{type(exception).__name__}: {exception}",
            )
            self._with_span.record_exception(exception)
            self._finish_tracing(status=status)
            raise
        # completed without exception
        status = trace_api.Status(
            status_code=trace_api.StatusCode.OK,
        )
        self._finish_tracing(status=status)

    async def aclose(self):
        await self.__wrapped__.aclose()
        if not self._is_finished:
            self._is_finished = True
            status = trace_api.Status(status_code=trace_api.StatusCode.OK)
            self._finish_tracing(status=status)

    def __del__(self):
        if not getattr(self, "_is_finished", False):
            self._is_finished = True
            status = trace_api.Status(status_code=trace_api.StatusCode.OK)
            self._finish_tracing(status=status)

    def _finish_tracing(
        self,
        status: Optional[trace_api.Status] = None,
    ) -> None:
        _finish_tracing(
            with_span=self._with_span,
            has_attributes=_MessageResponseExtractor(
                response_accumulator=self._response_accumulator
            ),
            status=status,
        )


class _MessageResponseAccumulator:
    __slots__ = (
        "_is_null",
        "_values",
        "_current_message_idx",
        "_current_content_block_type",
    )

    def __init__(self) -> None:
        self._is_null = True
        self._current_message_idx = -1
        self._current_content_block_type: Union[
            "TextBlock", "ToolUseBlock", None
        ] = None
        self._values = _ValuesAccumulator(
            messages=_IndexedAccumulator(
                lambda: _ValuesAccumulator(
                    role=_SimpleStringReplace(),
                    content=_IndexedAccumulator(
                        lambda: _ValuesAccumulator(
                            type=_SimpleStringReplace(),
                            text=_StringAccumulator(),
                            tool_name=_SimpleStringReplace(),
                            tool_input=_StringAccumulator(),
                        ),
                    ),
                    stop_reason=_SimpleStringReplace(),
                    input_tokens=_SimpleStringReplace(),
                    output_tokens=_SimpleStringReplace(),
                ),
            ),
        )

    def process_chunk(self, chunk: "RawContentBlockDeltaEvent") -> None:
        from anthropic.types.raw_content_block_delta_event import (
            RawContentBlockDeltaEvent,
        )
        from anthropic.types.raw_content_block_start_event import (
            RawContentBlockStartEvent,
        )
        from anthropic.types.raw_message_delta_event import RawMessageDeltaEvent
        from anthropic.types.raw_message_start_event import RawMessageStartEvent
        from anthropic.types.text_block import TextBlock
        from anthropic.types.tool_use_block import ToolUseBlock

        self._is_null = False
        if isinstance(chunk, RawMessageStartEvent):
            self._current_message_idx += 1
            value = {
                "messages": {
                    "index": str(self._current_message_idx),
                    "role": chunk.message.role,
                    "input_tokens": str(chunk.message.usage.input_tokens),
                }
            }
            self._values += value
        elif isinstance(chunk, RawContentBlockStartEvent):
            self._current_content_block_type = chunk.content_block
        elif isinstance(chunk, RawContentBlockDeltaEvent):
            if isinstance(self._current_content_block_type, TextBlock):
                value = {
                    "messages": {
                        "index": str(self._current_message_idx),
                        "content": {
                            "index": chunk.index,
                            "type": self._current_content_block_type.type,
                            "text": chunk.delta.text,  # type: ignore
                        },
                    }
                }
                self._values += value
            elif isinstance(self._current_content_block_type, ToolUseBlock):
                value = {
                    "messages": {
                        "index": str(self._current_message_idx),
                        "content": {
                            "index": chunk.index,
                            "type": self._current_content_block_type.type,
                            "tool_name": self._current_content_block_type.name,
                            "tool_input": chunk.delta.partial_json,  # type: ignore
                        },
                    }
                }
                self._values += value
        elif isinstance(chunk, RawMessageDeltaEvent):
            value = {
                "messages": {
                    "index": str(self._current_message_idx),
                    "stop_reason": chunk.delta.stop_reason,
                    "output_tokens": str(chunk.usage.output_tokens),
                }
            }
            self._values += value

    def _result(self) -> Optional[Dict[str, Any]]:
        if self._is_null:
            return None
        return dict(self._values)


class _MessageResponseExtractor:
    __slots__ = ("_response_accumulator",)

    def __init__(
        self,
        response_accumulator: _MessageResponseAccumulator,
    ) -> None:
        self._response_accumulator = response_accumulator

    def get_attributes(self) -> Iterator[Tuple[str, AttributeValue]]:
        if not (result := self._response_accumulator._result()):
            return

        json_string = safe_json_dumps(result)
        yield SpanAttributes.RAW_OUTPUT, json_string

        messages = result.get("messages", [])
        message_content = ""
        for message in messages:
            if message.get("role") == "assistant":
                for content_block in message.get("content", []):
                    if text := content_block.get("text"):
                        message_content += text
        if message_content:
            yield from _as_output_attributes(
                _ValueAndType(message_content, FiMimeTypeValues.TEXT)
            )
        else:
            json_string = safe_json_dumps(result)
            yield from _as_output_attributes(
                _ValueAndType(json_string, FiMimeTypeValues.JSON)
            )

    def get_extra_attributes(self) -> Iterator[Tuple[str, AttributeValue]]:
        if not (result := self._response_accumulator._result()):
            return
        messages = result.get("messages", [])
        idx = 0
        total_completion_token_count = 0
        total_prompt_token_count = 0
        # TODO(harrison): figure out if we should always assume messages is 1.
        # The current non streaming implementation assumes the same
        for message in messages:
            if role := message.get("role"):
                yield (
                    f"{SpanAttributes.LLM_OUTPUT_MESSAGES}.{idx}.{MessageAttributes.MESSAGE_ROLE}",
                    role,
                )
            if output_tokens := message.get("output_tokens"):
                total_completion_token_count += int(output_tokens)
            if input_tokens := message.get("input_tokens"):
                total_prompt_token_count += int(input_tokens)

            # TODO(harrison): figure out if we should always assume the first message
            #  will always be a message output generally this block feels really
            #  brittle to imitate the current non streaming implementation.
            tool_idx = 0
            for content in message.get("content", []):
                # this is the current assumption of the non streaming implementation.
                if (content_type := content.get("type")) == "text":
                    yield (
                        f"{SpanAttributes.LLM_OUTPUT_MESSAGES}.{idx}.{MessageAttributes.MESSAGE_CONTENT}",
                        content.get("text", ""),
                    )
                elif content_type == "tool_use":
                    yield (
                        f"{SpanAttributes.LLM_OUTPUT_MESSAGES}.{idx}.{MessageAttributes.MESSAGE_TOOL_CALLS}.{tool_idx}.{ToolCallAttributes.TOOL_CALL_FUNCTION_NAME}",
                        content.get("tool_name", ""),
                    )
                    yield (
                        f"{SpanAttributes.LLM_OUTPUT_MESSAGES}.{idx}.{MessageAttributes.MESSAGE_TOOL_CALLS}.{tool_idx}.{ToolCallAttributes.TOOL_CALL_FUNCTION_ARGUMENTS_JSON}",
                        content.get("tool_input", "{}"),
                    )
                    tool_idx += 1
            idx += 1
        yield SpanAttributes.LLM_TOKEN_COUNT_COMPLETION, total_completion_token_count
        yield SpanAttributes.LLM_TOKEN_COUNT_PROMPT, total_prompt_token_count
        yield (
            SpanAttributes.LLM_TOKEN_COUNT_TOTAL,
            total_completion_token_count + total_prompt_token_count,
        )


class _ValuesAccumulator:
    __slots__ = ("_values",)

    def __init__(self, **values: Any) -> None:
        self._values: Dict[str, Any] = values

    def __iter__(self) -> Iterator[Tuple[str, Any]]:
        for key, value in self._values.items():
            if value is None:
                continue
            if isinstance(value, _ValuesAccumulator):
                if dict_value := dict(value):
                    yield key, dict_value
            elif isinstance(value, _IndexedAccumulator):
                list_value = list(value)
                if list_value:
                    yield key, list_value
            elif isinstance(value, _SimpleStringReplace):
                if str_value := str(value):
                    yield key, str_value
            elif isinstance(value, _StringAccumulator):
                if str_value := str(value):
                    yield key, str_value
            else:
                yield key, value

    def __iadd__(self, values: Optional[Mapping[str, Any]]) -> "_ValuesAccumulator":
        if not values:
            return self
        for key in self._values.keys():
            if (value := values.get(key)) is None:
                continue
            self_value = self._values[key]
            if isinstance(self_value, _ValuesAccumulator):
                if isinstance(value, Mapping):
                    self_value += value
            elif isinstance(self_value, _StringAccumulator):
                if isinstance(value, str):
                    self_value += value
            elif isinstance(self_value, _SimpleStringReplace):
                if isinstance(value, str):
                    self_value += value
            elif isinstance(self_value, _IndexedAccumulator):
                self_value += value
            elif isinstance(self_value, List) and isinstance(value, Iterable):
                self_value.extend(value)
            else:
                self._values[key] = value  # replacement
        for key in values.keys():
            if key in self._values or (value := values[key]) is None:
                continue
            value = deepcopy(value)
            if isinstance(value, Mapping):
                value = _ValuesAccumulator(**value)
            self._values[key] = value  # new entry
        return self


class _StringAccumulator:
    __slots__ = ("_fragments",)

    def __init__(self) -> None:
        self._fragments: List[str] = []

    def __str__(self) -> str:
        return "".join(self._fragments)

    def __iadd__(self, value: Optional[str]) -> "_StringAccumulator":
        if not value:
            return self
        self._fragments.append(value)
        return self


class _IndexedAccumulator:
    __slots__ = ("_indexed",)

    def __init__(self, factory: Callable[[], _ValuesAccumulator]) -> None:
        self._indexed: DefaultDict[int, _ValuesAccumulator] = defaultdict(factory)

    def __iter__(self) -> Iterator[Dict[str, Any]]:
        for _, values in sorted(self._indexed.items()):
            yield dict(values)

    def __iadd__(self, values: Optional[Mapping[str, Any]]) -> "_IndexedAccumulator":
        if (
            not values
            or not hasattr(values, "get")
            or (index := values.get("index")) is None
        ):
            return self
        self._indexed[index] += values
        return self


class _SimpleStringReplace:
    __slots__ = ("_str_val",)

    def __init__(self) -> None:
        self._str_val: str = ""

    def __str__(self) -> str:
        return self._str_val

    def __iadd__(self, value: Optional[str]) -> "_SimpleStringReplace":
        if not value:
            return self
        self._str_val = value
        return self



import logging
from typing import Any, Dict, Optional, Union

from opentelemetry import trace as trace_api
from opentelemetry.util.types import Attributes, AttributeValue

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class _WithSpan:
    __slots__ = (
        "_span",
        "_is_finished",
    )

    def __init__(
        self,
        span: trace_api.Span,
    ) -> None:
        self._span = span
        try:
            self._is_finished = not self._span.is_recording()
        except Exception:
            logger.exception("Failed to check if span is recording")
            print("Error checking span recording status, defaulting to finished")
            self._is_finished = True

    @property
    def is_finished(self) -> bool:
        return self._is_finished

    def set_attribute(self, key: str, value: Any) -> None:
        self._span.set_attribute(key, value)

    def set_attributes(self, attributes: Dict[str, AttributeValue]) -> None:
        self._span.set_attributes(attributes)

    def record_exception(self, exception: Exception) -> None:
        if self._is_finished:
            return
        try:
            self._span.record_exception(exception)
        except Exception:
            logger.exception("Failed to record exception on span")
            print("Error recording exception on span")

    def set_status(self, status: Union[trace_api.Status, trace_api.StatusCode]) -> None:
        if self._is_finished:
            return
        try:
            self._span.set_status(status=status)
        except Exception:
            logger.exception("Failed to set status on span")
            print("Error setting status on span")

    def add_event(self, name: str) -> None:
        if self._is_finished:
            return
        try:
            self._span.add_event(name)
        except Exception:
            logger.exception("Failed to add event to span")
            print("Error adding event to span")

    def finish_tracing(
        self,
        status: Optional[trace_api.Status] = None,
        attributes: Attributes = None,
        extra_attributes: Attributes = None,
    ) -> None:
        if self._is_finished:
            return

        for mapping in (attributes, extra_attributes):
            if not mapping:
                continue
            for key, value in mapping.items():
                if value is None:
                    continue
                try:
                    self._span.set_attribute(key, value)
                except Exception:
                    logger.exception("Failed to set attribute on span")
                    print(f"Error setting attribute: {key}")

        if status is not None:
            try:
                self._span.set_status(status=status)
            except Exception:
                logger.exception("Failed to set status code on span")
                print("Error setting final status")

        try:
            self._span.end()
        except Exception:
            logger.exception("Failed to end span")
            print("Error ending span")

        self._is_finished = True



from abc import ABC
from contextlib import contextmanager
from itertools import chain
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Dict,
    Iterator,
    List,
    Mapping,
    Optional,
    Tuple,
)

import opentelemetry.context as context_api
from anthropic.types import Message, MessageTokensCount
from fi_instrumentation import get_attributes_from_context, safe_json_dumps
from fi_instrumentation.fi_types import (
    DocumentAttributes,
    EmbeddingAttributes,
    FiLLMProviderValues,
    FiLLMSystemValues,
    FiMimeTypeValues,
    FiSpanKindValues,
    MessageAttributes,
    MessageContentAttributes,
    SpanAttributes,
    ToolAttributes,
    ToolCallAttributes,
)
from opentelemetry import trace as trace_api
from opentelemetry.trace import INVALID_SPAN
from traceai_anthropic._stream import _MessagesStream, _Stream
from traceai_anthropic._with_span import _WithSpan

if TYPE_CHECKING:
    from anthropic.types import Usage
    from pydantic import BaseModel


class _WithTracer(ABC):
    """
    Base class for wrappers that need a tracer.
    """

    def __init__(self, tracer: trace_api.Tracer, *args: Any, **kwargs: Any) -> None:
        print(f"Initializing _WithTracer with tracer: {tracer}")
        super().__init__(*args, **kwargs)
        self._tracer = tracer

    @contextmanager
    def _start_as_current_span(
        self, span_name: str, attributes: Optional[Mapping[str, Any]] = None
    ) -> Iterator[_WithSpan]:
        try:
            span = self._tracer.start_span(
                name=span_name,
                record_exception=False,
                set_status_on_exception=False,
                attributes=attributes,
            )
        except Exception as e:
            print(f"Error creating span: {e}")
            span = INVALID_SPAN
        with trace_api.use_span(
            span,
            end_on_exit=False,
            record_exception=False,
            set_status_on_exception=False,
        ) as span:
            yield _WithSpan(span=span)


class _BaseWrapper(_WithTracer):
    """Base wrapper class with common functionality"""

    def _add_io_to_span_attributes(
        self,
        span: _WithSpan,
        input_data: Optional[Any] = None,
        output_data: Optional[Any] = None,
    ) -> None:
        """Add input/output data to span attributes"""
        try:
            if input_data is not None:
                span.set_attributes({"fi.llm.input": str(input_data)})
            if output_data is not None:
                span.set_attributes({"fi.llm.output": str(output_data)})
        except Exception as e:
            print(f"Error adding I/O to span attributes: {e}")


class _CompletionsWrapper(_BaseWrapper):
    """
    Wrapper for the pipeline processing
    Captures all calls to the pipeline
    """

    __slots__ = "_response_accumulator"

    def __call__(
        self,
        wrapped: Callable[..., Any],
        instance: Any,
        args: Tuple[Any, ...],
        kwargs: Mapping[str, Any],
    ) -> Any:
        if context_api.get_value(context_api._SUPPRESS_INSTRUMENTATION_KEY):
            return wrapped(*args, **kwargs)

        arguments = kwargs
        llm_prompt = dict(arguments).pop("system", None)
        llm_input_messages = dict(arguments).pop("messages", None)
        llm_invocation_parameters = _get_invocation_parameters(arguments)

        extracted_data = _extract_image_data(llm_input_messages)
        llm_input_messages = extracted_data["filtered_messages"]
        input_images = extracted_data["input_images"]
        eval_input = extracted_data["eval_input"]
        query = extracted_data["query"]

        with self._start_as_current_span(
            span_name="Completions",
            attributes=dict(
                chain(
                    get_attributes_from_context(),
                    _get_llm_model_name_from_input(arguments),
                    _get_llm_provider(),
                    _get_llm_system(),
                    _get_llm_span_kind(),
                    _get_llm_prompts(llm_prompt),
                    _get_inputs(llm_input_messages),
                    _get_llm_invocation_parameters(llm_invocation_parameters),
                    _get_llm_tools(llm_invocation_parameters),
                    _get_image_inputs(input_images),
                    _get_eval_input(eval_input),
                    _get_raw_input(arguments),
                    _get_query(query),
                )
            ),
        ) as span:
            try:
                response = wrapped(*args, **kwargs)
                span.set_attributes(dict(_get_raw_output(response)))

                # Add I/O to span attributes instead of sending separately
                self._add_io_to_span_attributes(
                    span=span,
                    output_data=(
                        response.completion if hasattr(response, "completion") else None
                    ),
                )

            except Exception as exception:
                print(f"Error in completion: {exception}")
                span.set_status(
                    trace_api.Status(trace_api.StatusCode.ERROR, str(exception))
                )
                span.record_exception(exception)
                raise

            streaming = kwargs.get("stream", False)
            if streaming:
                return _Stream(response, span)
            else:
                span.set_status(trace_api.StatusCode.OK)
                span.set_attributes(dict(_get_outputs(response)))
                span.finish_tracing()
                return response


class _AsyncCompletionsWrapper(_WithTracer):
    """
    Wrapper for the pipeline processing
    Captures all calls to the pipeline
    """

    async def __call__(
        self,
        wrapped: Callable[..., Any],
        instance: Any,
        args: Tuple[Any, ...],
        kwargs: Mapping[str, Any],
    ) -> Any:
        if context_api.get_value(context_api._SUPPRESS_INSTRUMENTATION_KEY):
            return await wrapped(*args, **kwargs)

        arguments = kwargs
        llm_prompt = dict(arguments).pop("system", None)
        llm_input_messages = dict(arguments).pop("messages", None)
        invocation_parameters = _get_invocation_parameters(arguments)

        extracted_data = _extract_image_data(llm_input_messages)
        llm_input_messages = extracted_data["filtered_messages"]
        input_images = extracted_data["input_images"]
        eval_input = extracted_data["eval_input"]
        query = extracted_data["query"]

        with self._start_as_current_span(
            span_name="AsyncCompletions",
            attributes=dict(
                chain(
                    get_attributes_from_context(),
                    _get_llm_model_name_from_input(arguments),
                    _get_llm_provider(),
                    _get_llm_system(),
                    _get_llm_span_kind(),
                    _get_llm_prompts(llm_prompt),
                    _get_inputs(llm_input_messages),
                    _get_llm_invocation_parameters(invocation_parameters),
                    _get_llm_tools(invocation_parameters),
                    _get_image_inputs(input_images),
                    _get_eval_input(eval_input),
                    _get_raw_input(arguments),
                    _get_query(query),
                )
            ),
        ) as span:
            try:
                response = await wrapped(*args, **kwargs)
                span.set_attributes(dict(_get_raw_output(response)))
            except Exception as exception:
                span.set_status(
                    trace_api.Status(trace_api.StatusCode.ERROR, str(exception))
                )
                span.record_exception(exception)
                raise
            span.set_status(trace_api.StatusCode.OK)
            streaming = kwargs.get("stream", False)
            if streaming:
                return _Stream(response, span)
            else:
                span.set_status(trace_api.StatusCode.OK)
                span.set_attributes(dict(_get_async_outputs(response)))
                span.finish_tracing()
                return response


class _MessagesWrapper(_BaseWrapper):
    """
    Wrapper for the pipeline processing
    Captures all calls to the pipeline
    """

    def __call__(
        self,
        wrapped: Callable[..., Any],
        instance: Any,
        args: Tuple[Any, ...],
        kwargs: Mapping[str, Any],
    ) -> Any:
        if context_api.get_value(context_api._SUPPRESS_INSTRUMENTATION_KEY):
            return wrapped(*args, **kwargs)

        arguments = kwargs
        llm_input_messages = dict(arguments).pop("messages", None)
        llm_prompt = dict(arguments).pop("system", None)
        invocation_parameters = _get_invocation_parameters(arguments)

        extracted_data = _extract_image_data(llm_input_messages)
        llm_input_messages = extracted_data["filtered_messages"]
        input_images = extracted_data["input_images"]
        eval_input = extracted_data["eval_input"]
        query = extracted_data["query"]

        with self._start_as_current_span(
            span_name="Messages",
            attributes=dict(
                chain(
                    get_attributes_from_context(),
                    _get_llm_model_name_from_input(arguments),
                    _get_llm_provider(),
                    _get_llm_system(),
                    _get_llm_span_kind(),
                    _get_llm_prompts(llm_prompt),
                    _get_llm_input_messages(llm_input_messages),
                    _get_llm_invocation_parameters(invocation_parameters),
                    _get_llm_tools(invocation_parameters),
                    _get_inputs(llm_input_messages),
                    _get_image_inputs(input_images),
                    _get_eval_input(eval_input),
                    _get_raw_input(arguments),
                    _get_query(query),
                )
            ),
        ) as span:
            try:
                response = wrapped(*args, **kwargs)
                streaming = kwargs.get("stream", False)
                if not streaming:
                    span.set_attributes(dict(_get_raw_output(response)))

                # Get output content
                output_content = None
                if hasattr(response, "content"):
                    if isinstance(response.content, list):
                        for block in response.content:
                            if hasattr(block, "text"):
                                output_content = block.text
                                break
                            elif isinstance(block, dict) and "text" in block:
                                output_content = block["text"]
                                break

                # Add I/O to span attributes instead of sending separately
                self._add_io_to_span_attributes(span=span, output_data=output_content)

            except Exception as exception:
                span.set_status(
                    trace_api.Status(trace_api.StatusCode.ERROR, str(exception))
                )
                span.record_exception(exception)
                raise

            streaming = kwargs.get("stream", False)
            if streaming:
                return _MessagesStream(response, span)
            else:
                span.set_status(trace_api.StatusCode.OK)
                span.set_attributes(
                    dict(
                        chain(
                            _get_llm_model_name_from_response(response),
                            _get_output_messages(response),
                            _get_llm_token_counts(response.usage),
                            _get_outputs(response),
                        )
                    )
                )
                span.finish_tracing()
                return response


class _AsyncMessagesWrapper(_WithTracer):
    """
    Wrapper for the pipeline processing
    Captures all calls to the pipeline
    """

    async def __call__(
        self,
        wrapped: Callable[..., Any],
        instance: Any,
        args: Tuple[Any, ...],
        kwargs: Mapping[str, Any],
    ) -> Any:
        if context_api.get_value(context_api._SUPPRESS_INSTRUMENTATION_KEY):
            return await wrapped(*args, **kwargs)

        arguments = kwargs
        llm_prompt = dict(arguments).pop("system", None)
        llm_input_messages = dict(arguments).pop("messages", None)
        invocation_parameters = _get_invocation_parameters(arguments)

        extracted_data = _extract_image_data(llm_input_messages)
        llm_input_messages = extracted_data["filtered_messages"]
        input_images = extracted_data["input_images"]
        eval_input = extracted_data["eval_input"]
        query = extracted_data["query"]

        with self._start_as_current_span(
            span_name="AsyncMessages",
            attributes=dict(
                chain(
                    get_attributes_from_context(),
                    _get_llm_provider(),
                    _get_llm_system(),
                    _get_llm_model_name_from_input(arguments),
                    _get_llm_span_kind(),
                    _get_llm_prompts(llm_prompt),
                    _get_llm_input_messages(llm_input_messages),
                    _get_llm_invocation_parameters(invocation_parameters),
                    _get_llm_tools(invocation_parameters),
                    _get_inputs(llm_input_messages),
                    _get_image_inputs(input_images),
                    _get_eval_input(eval_input),
                    _get_raw_input(arguments),
                    _get_query(query),
                )
            ),
        ) as span:
            try:
                response = await wrapped(*args, **kwargs)
                span.set_attributes(dict(_get_raw_output(response)))
            except Exception as exception:
                span.set_status(
                    trace_api.Status(trace_api.StatusCode.ERROR, str(exception))
                )
                span.record_exception(exception)
                raise
            streaming = kwargs.get("stream", False)
            if streaming:
                return _MessagesStream(response, span)
            else:
                span.set_status(trace_api.StatusCode.OK)
                span.set_attributes(
                    dict(
                        chain(
                            _get_llm_model_name_from_response(response),
                            _get_output_messages(response),
                            _get_llm_token_counts(response.usage),
                            _get_async_outputs(response),
                        )
                    )
                )
                span.finish_tracing()
                return response


class _MessagesCountTokensWrapper(_BaseWrapper):
    """
    Wrapper for the Messages.count_tokens method to create spans.
    """

    def __call__(
        self,
        wrapped,
        instance,
        args,
        kwargs,
    ):
        if context_api.get_value(context_api._SUPPRESS_INSTRUMENTATION_KEY):
            return wrapped(*args, **kwargs)

        arguments = kwargs
        llm_input_messages = arguments.get("messages", None)
        llm_prompt = dict(arguments).pop("system", None)

        extracted_data = _extract_image_data(llm_input_messages)
        llm_input_messages = extracted_data["filtered_messages"]
        input_images = extracted_data["input_images"]
        eval_input = extracted_data["eval_input"]
        query = extracted_data["query"]

        with self._start_as_current_span(
            span_name="MessagesCountTokens",
            attributes=dict(
                chain(
                    get_attributes_from_context(),
                    _get_llm_model_name_from_input(arguments),
                    _get_llm_provider(),
                    _get_llm_system(),
                    _get_llm_span_kind(),
                    _get_llm_prompts(llm_prompt),
                    _get_llm_input_messages(llm_input_messages),
                    _get_inputs(llm_input_messages),
                    _get_image_inputs(input_images),
                    _get_eval_input(eval_input),
                    _get_raw_input(arguments),
                    _get_query(query),
                )
            ),
        ) as span:
            try:
                response = wrapped(*args, **kwargs)
                span.set_attributes(dict(_get_raw_output(response)))
            except Exception as exception:
                span.set_status(
                    trace_api.Status(trace_api.StatusCode.ERROR, str(exception))
                )
                span.record_exception(exception)
                raise

            span.set_status(trace_api.StatusCode.OK)

            output_data = getattr(response, "input_tokens", None)

            self._add_io_to_span_attributes(span=span, output_data=output_data)

            if output_data is not None:
                span.set_attribute(SpanAttributes.LLM_TOKEN_COUNT_PROMPT, output_data)
            span.finish_tracing()
            return response


def _get_inputs(arguments: Mapping[str, Any]) -> Iterator[Tuple[str, Any]]:
    yield INPUT_VALUE, safe_json_dumps(arguments)
    yield INPUT_MIME_TYPE, JSON


def _get_raw_input(arguments: Mapping[str, Any]) -> Iterator[Tuple[str, Any]]:
    yield RAW_INPUT, safe_json_dumps(arguments)


def _get_raw_output(response) -> Iterator[Tuple[str, Any]]:
    if isinstance(response, (Message, MessageTokensCount)):
        yield RAW_OUTPUT, safe_json_dumps(response.to_dict())
    elif hasattr(response, "to_dict"):
        yield RAW_OUTPUT, safe_json_dumps(response.to_dict())
    else:
        yield RAW_OUTPUT, safe_json_dumps(response)


def _get_query(query: str) -> Iterator[Tuple[str, Any]]:
    yield QUERY, query


def _get_response(response: Any) -> Iterator[Tuple[str, Any]]:
    yield RESPONSE, response


def class_to_dict(obj: Any) -> Dict[str, Any]:
    """
    Recursively convert a class instance to a dictionary.
    """
    if isinstance(obj, dict):
        return {k: class_to_dict(v) for k, v in obj.items()}
    elif isinstance(obj, list):
        return [class_to_dict(v) for v in obj]
    elif hasattr(obj, "__dict__"):
        return {k: class_to_dict(v) for k, v in obj.__dict__.items()}
    else:
        return obj


def _extract_image_data(messages):
    try:
        input_images = []
        filtered_messages = []
        eval_input = []

        if isinstance(messages, list):
            for message in messages:
                filtered_content = []
                content = message.get("content", [])

                # Handle both string and list content
                if isinstance(content, str):
                    filtered_messages.append(message)
                    eval_input.append(content)
                    continue

                elif isinstance(content, list):
                    for item in content:
                        if isinstance(item, dict):
                            if item.get("type") == "image":
                                source = item.get("source", {})
                                if source.get("type") == "base64":
                                    data = source.get("data")
                                    if data:
                                        input_images.append(data)
                                        image_index = len(input_images) - 1
                                        eval_input.append(
                                            "{{"
                                            + f"{SpanAttributes.INPUT_IMAGES}.{image_index}"
                                            + "}}"
                                        )
                            else:
                                # Keep non-image content
                                filtered_content.append(item)
                                if item.get("type") == "text" and item.get("text"):
                                    eval_input.append(str(item.get("text")))

                # Create new message with filtered content
                if filtered_content:
                    filtered_message = message.copy()
                    filtered_message["content"] = filtered_content
                    filtered_messages.append(filtered_message)

        return {
            "input_images": input_images if input_images else None,
            "filtered_messages": filtered_messages if filtered_messages else None,
            "eval_input": "\n".join(eval_input),
            "query": str(eval_input[0]) if eval_input else None,
        }
    except Exception as e:
        print(f"Error in _extract_image_data: {e}")
        return {
            "input_images": None,
            "filtered_messages": messages,
            "eval_input": None,
            "query": None,
        }


def _get_image_inputs(input_images: Optional[List[str]]) -> Iterator[Tuple[str, Any]]:
    if input_images:
        yield SpanAttributes.INPUT_IMAGES, safe_json_dumps(input_images)


def _get_eval_input(input: Any) -> Iterator[Tuple[str, Any]]:
    if input is not None:
        yield SpanAttributes.EVAL_INPUT, safe_json_dumps(input)


def _get_outputs(response: "BaseModel") -> Iterator[Tuple[str, Any]]:
    yield OUTPUT_VALUE, response.model_dump_json()
    yield OUTPUT_MIME_TYPE, JSON


def _get_async_outputs(response: "BaseModel") -> Iterator[Tuple[str, Any]]:
    output_value = response
    if hasattr(response, "content"):
        content = response.content
        if isinstance(content, list):
            for block in content:
                if hasattr(block, "text"):
                    output_value = block.text
                    break
                elif isinstance(block, dict) and "text" in block:
                    output_value = block["text"]
                    break

    yield OUTPUT_VALUE, output_value
    yield OUTPUT_MIME_TYPE, JSON


def _get_llm_tools(
    invocation_parameters: Mapping[str, Any]
) -> Iterator[Tuple[str, Any]]:
    if isinstance(tools := invocation_parameters.get("tools"), list):
        yield LLM_TOOLS, safe_json_dumps(tools)
        for tool_index, tool_schema in enumerate(tools):
            yield f"{LLM_TOOLS}.{tool_index}.{TOOL_JSON_SCHEMA}", safe_json_dumps(
                tool_schema
            )


def _get_llm_span_kind() -> Iterator[Tuple[str, Any]]:
    yield FI_SPAN_KIND, LLM


def _get_llm_provider() -> Iterator[Tuple[str, Any]]:
    yield LLM_PROVIDER, LLM_PROVIDER_ANTHROPIC


def _get_llm_system() -> Iterator[Tuple[str, Any]]:
    yield LLM_SYSTEM, LLM_SYSTEM_ANTHROPIC


def _get_llm_token_counts(usage: "Usage") -> Iterator[Tuple[str, Any]]:
    yield LLM_TOKEN_COUNT_PROMPT, usage.input_tokens
    yield LLM_TOKEN_COUNT_COMPLETION, usage.output_tokens
    yield LLM_TOKEN_COUNT_TOTAL, usage.input_tokens + usage.output_tokens


def _get_llm_model_name_from_input(
    arguments: Mapping[str, Any]
) -> Iterator[Tuple[str, Any]]:
    if model_name := arguments.get("model"):
        yield LLM_MODEL_NAME, model_name


def _get_llm_model_name_from_response(message: "Message") -> Iterator[Tuple[str, Any]]:
    if model_name := getattr(message, "model"):
        yield LLM_MODEL_NAME, model_name


def _get_llm_invocation_parameters(
    invocation_parameters: Mapping[str, Any],
) -> Iterator[Tuple[str, Any]]:
    yield LLM_INVOCATION_PARAMETERS, safe_json_dumps(invocation_parameters)


def _get_llm_prompts(prompt: str) -> Iterator[Tuple[str, Any]]:
    yield LLM_PROMPTS, safe_json_dumps(prompt)


def _get_llm_input_messages(messages: List[Dict[str, str]]) -> Any:
    from anthropic.types import TextBlock, ToolUseBlock

    for i in range(len(messages)):
        tool_index = 0
        if content := messages[i].get("content"):
            if isinstance(content, str):
                yield f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_CONTENT}", content
            elif isinstance(content, list):
                for block_index, block in enumerate(content):
                    if isinstance(block, ToolUseBlock):
                        if tool_call_id := block.id:
                            yield (
                                f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_TOOL_CALLS}.{tool_index}.{TOOL_CALL_ID}",
                                tool_call_id,
                            )
                        yield (
                            f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_TOOL_CALLS}.{tool_index}.{TOOL_CALL_FUNCTION_NAME}",
                            block.name,
                        )
                        yield (
                            f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_TOOL_CALLS}.{tool_index}.{TOOL_CALL_FUNCTION_ARGUMENTS_JSON}",
                            safe_json_dumps(block.input),
                        )
                        tool_index += 1
                    elif isinstance(block, TextBlock):
                        yield f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_CONTENT}.{block_index}.{MESSAGE_CONTENT_TYPE}", "text"
                        yield f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_CONTENT}.{block_index}.{MESSAGE_CONTENT_TEXT}", block.text
                    elif isinstance(block, dict):
                        block_type = block.get("type")
                        if block_type == "tool_use":
                            if (tool_call_id := block.get("id")) is not None:
                                yield (
                                    f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_TOOL_CALLS}.{tool_index}.{TOOL_CALL_ID}",
                                    str(tool_call_id),
                                )
                            yield (
                                f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_TOOL_CALLS}.{tool_index}.{TOOL_CALL_FUNCTION_NAME}",
                                block.get("name"),
                            )
                            yield (
                                f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_TOOL_CALLS}.{tool_index}.{TOOL_CALL_FUNCTION_ARGUMENTS_JSON}",
                                safe_json_dumps(block.get("input")),
                            )
                            tool_index += 1
                        elif block_type == "tool_result":
                            if (tool_call_id := block.get("tool_use_id")) is not None:
                                yield (
                                    f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_TOOL_CALL_ID}",
                                    str(tool_call_id),
                                )
                            if (content := block.get("content")) is not None:
                                yield (
                                    f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_CONTENT}",
                                    (
                                        content
                                        if isinstance(content, str)
                                        else safe_json_dumps(content)
                                    ),
                                )
                        elif block_type == "text":
                            yield f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_CONTENT}.{block_index}.{MESSAGE_CONTENT_TYPE}", "text"
                            yield f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_CONTENT}.{block_index}.{MESSAGE_CONTENT_TEXT}", block.get(
                                "text"
                            )

        if role := messages[i].get("role"):
            yield f"{LLM_INPUT_MESSAGES}.{i}.{MESSAGE_ROLE}", role


def _get_invocation_parameters(kwargs: Mapping[str, Any]) -> Any:
    invocation_parameters = {}
    for key, value in kwargs.items():
        if _validate_invocation_parameter(key):
            invocation_parameters[key] = value
    return invocation_parameters


def _get_output_messages(response: Any) -> Any:
    """
    Extracts the tool call information from the response
    """
    from anthropic.types import TextBlock, ToolUseBlock

    tool_index = 0
    for block in response.content:
        yield f"{LLM_OUTPUT_MESSAGES}.{0}.{MESSAGE_ROLE}", response.role
        if isinstance(block, ToolUseBlock):
            yield (
                f"{LLM_OUTPUT_MESSAGES}.{0}.{MESSAGE_TOOL_CALLS}.{tool_index}.{TOOL_CALL_FUNCTION_NAME}",
                block.name,
            )
            yield (
                f"{LLM_OUTPUT_MESSAGES}.{0}.{MESSAGE_TOOL_CALLS}.{tool_index}.{TOOL_CALL_FUNCTION_ARGUMENTS_JSON}",
                safe_json_dumps(block.input),
            )
            tool_index += 1
        if isinstance(block, TextBlock):
            yield f"{LLM_OUTPUT_MESSAGES}.{0}.{MESSAGE_CONTENT}", block.text


def _validate_invocation_parameter(parameter: Any) -> bool:
    """
    Validates the invocation parameters.
    """
    valid_params = (
        "max_tokens",
        "max_tokens_to_sample",
        "model",
        "metadata",
        "stop_sequences",
        "stream",
        "system",
        "temperature",
        "tool_choice",
        "tools",
        "top_k",
        "top_p",
    )

    return parameter in valid_params


CHAIN = FiSpanKindValues.CHAIN.value
EMBEDDING = FiSpanKindValues.EMBEDDING.value
LLM = FiSpanKindValues.LLM.value
RETRIEVER = FiSpanKindValues.RETRIEVER.value

JSON = FiMimeTypeValues.JSON.value
TEXT = FiMimeTypeValues.TEXT.value

DOCUMENT_CONTENT = DocumentAttributes.DOCUMENT_CONTENT
DOCUMENT_ID = DocumentAttributes.DOCUMENT_ID
DOCUMENT_SCORE = DocumentAttributes.DOCUMENT_SCORE
DOCUMENT_METADATA = DocumentAttributes.DOCUMENT_METADATA
EMBEDDING_EMBEDDINGS = SpanAttributes.EMBEDDING_EMBEDDINGS
EMBEDDING_MODEL_NAME = SpanAttributes.EMBEDDING_MODEL_NAME
EMBEDDING_TEXT = EmbeddingAttributes.EMBEDDING_TEXT
EMBEDDING_VECTOR = EmbeddingAttributes.EMBEDDING_VECTOR
INPUT_MIME_TYPE = SpanAttributes.INPUT_MIME_TYPE
INPUT_VALUE = SpanAttributes.INPUT_VALUE
RAW_INPUT = SpanAttributes.RAW_INPUT
RAW_OUTPUT = SpanAttributes.RAW_OUTPUT
QUERY = SpanAttributes.QUERY
RESPONSE = SpanAttributes.RESPONSE
LLM_INPUT_MESSAGES = SpanAttributes.LLM_INPUT_MESSAGES
LLM_INVOCATION_PARAMETERS = SpanAttributes.LLM_INVOCATION_PARAMETERS
LLM_MODEL_NAME = SpanAttributes.LLM_MODEL_NAME
LLM_OUTPUT_MESSAGES = SpanAttributes.LLM_OUTPUT_MESSAGES
LLM_PROMPTS = SpanAttributes.LLM_PROMPTS
LLM_PROMPT_TEMPLATE = SpanAttributes.LLM_PROMPT_TEMPLATE
LLM_PROMPT_TEMPLATE_VARIABLES = SpanAttributes.LLM_PROMPT_TEMPLATE_VARIABLES
LLM_PROMPT_TEMPLATE_VERSION = SpanAttributes.LLM_PROMPT_TEMPLATE_VERSION
LLM_TOKEN_COUNT_COMPLETION = SpanAttributes.LLM_TOKEN_COUNT_COMPLETION
LLM_TOKEN_COUNT_PROMPT = SpanAttributes.LLM_TOKEN_COUNT_PROMPT
LLM_TOKEN_COUNT_TOTAL = SpanAttributes.LLM_TOKEN_COUNT_TOTAL
LLM_TOOLS = SpanAttributes.LLM_TOOLS
MESSAGE_CONTENT = MessageAttributes.MESSAGE_CONTENT
MESSAGE_CONTENTS = MessageAttributes.MESSAGE_CONTENTS
MESSAGE_FUNCTION_CALL_ARGUMENTS_JSON = (
    MessageAttributes.MESSAGE_FUNCTION_CALL_ARGUMENTS_JSON
)
MESSAGE_FUNCTION_CALL_NAME = MessageAttributes.MESSAGE_FUNCTION_CALL_NAME
MESSAGE_ROLE = MessageAttributes.MESSAGE_ROLE
MESSAGE_TOOL_CALLS = MessageAttributes.MESSAGE_TOOL_CALLS
MESSAGE_TOOL_CALL_ID = MessageAttributes.MESSAGE_TOOL_CALL_ID
METADATA = SpanAttributes.METADATA
FI_SPAN_KIND = SpanAttributes.FI_SPAN_KIND
OUTPUT_MIME_TYPE = SpanAttributes.OUTPUT_MIME_TYPE
OUTPUT_VALUE = SpanAttributes.OUTPUT_VALUE
RETRIEVAL_DOCUMENTS = SpanAttributes.RETRIEVAL_DOCUMENTS
SESSION_ID = SpanAttributes.SESSION_ID
TAG_TAGS = SpanAttributes.TAG_TAGS
TOOL_CALL_ID = ToolCallAttributes.TOOL_CALL_ID
TOOL_CALL_FUNCTION_ARGUMENTS_JSON = ToolCallAttributes.TOOL_CALL_FUNCTION_ARGUMENTS_JSON
TOOL_CALL_FUNCTION_NAME = ToolCallAttributes.TOOL_CALL_FUNCTION_NAME
TOOL_JSON_SCHEMA = ToolAttributes.TOOL_JSON_SCHEMA
USER_ID = SpanAttributes.USER_ID
LLM_PROVIDER = SpanAttributes.LLM_PROVIDER
LLM_SYSTEM = SpanAttributes.LLM_SYSTEM
LLM_PROVIDER_ANTHROPIC = FiLLMProviderValues.ANTHROPIC.value
LLM_SYSTEM_ANTHROPIC = FiLLMSystemValues.ANTHROPIC.value
MESSAGE_CONTENT_TEXT = MessageContentAttributes.MESSAGE_CONTENT_TEXT
MESSAGE_CONTENT_IMAGE = MessageContentAttributes.MESSAGE_CONTENT_IMAGE
MESSAGE_CONTENT_TYPE = MessageContentAttributes.MESSAGE_CONTENT_TYPE
